# imports
from bs4 import BeautifulSoup
import pandas as pd
from urllib.request import urlopen, Request
from os import path

HERE = path.abspath(path.dirname(__file__))


class ValueNotFound(Exception):
    """Exception raised when no value or list of values is given to the function.
    """

    def __init__(self, tickers, message="No value or list of values given to the function."):
        self.tickers = tickers
        self.message = message
        super().__init__(self.message)


class TickerNotFound(Exception):
    """Exception raised when the ticker given is not in the NASDAQ, AMEX or NYSE.
    """

    def __init__(self, tickers, message="Ticker not found in the NASDAQ, AMEX or NYSE."):
        self.tickers = tickers
        self.message = message
        super().__init__(self.message)


class InvalidDataType(Exception):
    """Exception raised when the ticker given is not a string or list.
    """

    def __init__(self, tickers, message="Invalid input data-type. Input should be string or list."):
        self.tickers = tickers
        self.message = message
        super().__init__(self.message)


with open(path.join(HERE, 'stock_list.txt')) as text_file:
    stock_list = [line.strip() for line in text_file.readlines()]

# text_file = open("stock_list.txt", 'r')
# stock_list = [line.strip() for line in text_file.readlines()]
web_url = 'https://finviz.com/quote.ashx?t='


def get_sentiment_data(tickers):
    """
    Returns dataframe of given ticker(s) with date, time, headlines and source.

    :param tickers: The ticker or list of tickers.
    :type tickers: str/list
    """
    if len(tickers) == 0 or tickers is None or len(str(tickers).strip()) == 0:
        raise ValueNotFound(tickers)

    if type(tickers) != str and type(tickers) != list:
        raise InvalidDataType(tickers)

    if type(tickers) == str:
        tickers = tickers.upper()
        if tickers not in stock_list:
            raise TickerNotFound(tickers)
        else:
            news_tables = {}
            url = web_url + tickers
            req = Request(url=url, headers={"User-Agent": "FireFox"})
            response = urlopen(req)
            html = BeautifulSoup(response, "html.parser")
            news_table = html.find(id='news-table')
            news_tables[tickers] = news_table
            news_list = []

            for file_name, news_table in news_tables.items():
                for i in news_table.findAll('tr'):

                    text = i.a.get_text()

                    date_scrape = i.td.text.split()

                    source = i.div.span.get_text()

                    if len(date_scrape) == 1:
                        time = date_scrape[0]

                    else:
                        date = date_scrape[0]
                        time = date_scrape[1]

                    tick = file_name.split('_')[0]

                    news_list.append([tick, date, time, source, text])
            columns = ['ticker', 'date', 'time', 'source', 'headline']
            news_df = pd.DataFrame(news_list, columns=columns)
            news_df['date'] = pd.to_datetime(news_df.date).dt.date

            return news_df

    if type(tickers) == list:
        tickers = [x.upper() for x in tickers]
        for x in tickers:
            if x not in stock_list:
                raise TickerNotFound(tickers)
        news_tables = {}
        for tick in tickers:
            url = web_url + tick
            req = Request(url=url, headers={"User-Agent": "FireFox"})
            response = urlopen(req)
            html = BeautifulSoup(response, "html.parser")
            news_table = html.find(id='news-table')
            news_tables[tick] = news_table
        news_list = []

        for file_name, news_table in news_tables.items():
            for i in news_table.findAll('tr'):

                text = i.a.get_text()

                date_scrape = i.td.text.split()
                source = i.div.span.get_text()

                if len(date_scrape) == 1:
                    time = date_scrape[0]

                else:
                    date = date_scrape[0]
                    time = date_scrape[1]

                tick = file_name.split('_')[0]

                news_list.append([tick, date, time, source, text])
        columns = ['ticker', 'date', 'time', 'source', 'headline']
        news_df = pd.DataFrame(news_list, columns=columns)
        news_df['date'] = pd.to_datetime(news_df.date).dt.date
        return news_df



# Always prefer setuptools over distutils
# To use a consistent encoding
from codecs import open
from os import path

from setuptools import setup

# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.md')) as f:
    long_description = f.read()

# This call to setup() does all the work
setup(
    name="stocksent",
    version="0.3.1",
    description="A Python library for sentiment analysis of various tickers from the latest news by trusted sources, "
                "and tools to plot results.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://stocksent.readthedocs.io/",
    author="Arya Manjaramkar",
    author_email="aryagm01@email.com",
    license="Mozilla Public License 2.0",
    classifiers=[
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Natural Language :: English",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent"
    ],
    packages=["stocksent"],
    include_package_data=True,
    install_requires=["numpy", "matplotlib",
                      "pandas", "nltk", "wordcloud", "bs4"]
)



import matplotlib.pyplot as plt
import nltk
import numpy as np
import pandas as pd
from nltk.sentiment.vader import SentimentIntensityAnalyzer
from datetime import datetime
from wordcloud import WordCloud

from stocksent.get_sentiment_data import get_sentiment_data

try:
    vader = SentimentIntensityAnalyzer()
except:
    nltk.download('vader_lexicon')
    vader = SentimentIntensityAnalyzer()


class Sentiment:
    """ 
    Instantiate ticker(s) for analysis.
    Provides several functions to analyze sentiments of stocks based on trusted news sources.
    
    :param ticker: The ticker or list of tickers.
    :type ticker: str/list
    """

    def __init__(self, ticker):
        self.ticker = ticker

    def get_sentiment(self, days=None):
        """
        Returns sentiment of given ticker(s).

        :param days: Number of days into the past you want the sentiment for. 
        :type days: int 
    
        """
        df = get_sentiment_data(self.ticker)
        if days is not None:
            now = datetime.now().date()
            day_neg = pd.to_timedelta(days, unit='d')
            oldest_date = now - day_neg
            df = df[df['date'] >= oldest_date]
        scores = df['headline'].apply(vader.polarity_scores).tolist()
        scores = [x['compound'] for x in scores]
        sentiment = float(np.mean(scores))
        final_sentiment = round(sentiment, 4)
        return final_sentiment

    def get_dataframe(self, days=None):
        """
        Returns dataframe of given ticker(s) with date, time, headlines, source and sentiment.

        :param days: Number of days into the past you want the sentiment for.
        :type days: int

        """
        df = get_sentiment_data(self.ticker)
        scores = df['headline'].apply(vader.polarity_scores).tolist()
        scores_df = pd.DataFrame(scores)
        scores_df.columns = ['Negative', 'Neutral', 'Positive', 'Overall']
        df = df.join(scores_df, rsuffix='_right')
        if days is not None:
            now = datetime.now().date()
            day_neg = pd.to_timedelta(days, unit='d')
            oldest_date = now - day_neg
            df = df[df['date'] >= oldest_date]
        return df

    def plot(self, days=None, save_figure=False):
        """
        Returns plot of given ticker(s) with sentiment across several days.

        :param days: Number of days into the past you want the plot for.
        :type days: int

        :param save_figure: Option for saving figure as png.
        :type save_figure: bool

        """
        df = get_sentiment_data(self.ticker)
        if days is not None:
            now = datetime.now().date()
            day_neg = pd.to_timedelta(days, unit='d')
            oldest_date = now - day_neg
            df = df[df['date'] >= oldest_date]
        scores = df['headline'].apply(vader.polarity_scores).tolist()
        scores_df = pd.DataFrame(scores)
        scores_df.columns = ['Negative', 'Neutral', 'Positive', 'Overall']
        df = df.join(scores_df, rsuffix='_right')

        plt.rcParams['figure.figsize'] = [10, 6]

        mean_scores = df.groupby(['ticker', 'date']).mean()
        mean_scores = mean_scores.unstack()
        mean_scores = mean_scores.xs('Overall', axis="columns").transpose()
        mean_scores.plot(kind='bar')

        plt.tight_layout()
        plt.grid()
        if save_figure:
            plt.savefig(str(datetime.now().timestamp()) + ".png")
        plt.show()
        pass

    def word_cloud(self, days=None, save_figure=False, figsize=(15, 8)):
        """
        Returns a word cloud plot of given ticker(s).

        :param days: Number of days into the past you want the word cloud for.
        :type days: int

        :param save_figure: Option for saving figure as png.
        :type save_figure: bool

        :param figsize: Option for resizing plot.
        :type save_figure: tuple

        """
        df = get_sentiment_data(self.ticker)
        if days is not None:
            now = datetime.now().date()
            day_neg = pd.to_timedelta(days, unit='d')
            oldest_date = now - day_neg
            df = df[df['date'] >= oldest_date]
        scores = df['headline'].apply(vader.polarity_scores).tolist()
        scores_df = pd.DataFrame(scores)
        scores_df.columns = ['Negative', 'Neutral', 'Positive', 'Overall']
        df = df.join(scores_df, rsuffix='_right')

        unique_string = " ".join(df['headline'].tolist())
        wordcloud = WordCloud(width=1000, height=500).generate(unique_string)

        plt.figure(figsize=figsize)
        plt.imshow(wordcloud)
        plt.axis("off")
        if save_figure:
            plt.savefig(str(datetime.now().timestamp()) + ".png")
        plt.show()
        pass
